from turms.processor.base import Processor


class BlackProcessor(Processor):
    def run(gen_file: str):
        from black import format_str, FileMode

        return format_str(gen_file, mode=FileMode())



from pydantic import BaseModel
from typing import List


class GeneratorConfig(BaseModel):
    generated_name: str = "generated.py"

    object_bases: List[str] = ["herre.access.newobject.GraphQLObject"]
    interface_bases: List[str] = ["herre.access.newobject.GraphQLObject"]

    scalar_definitions = {
        "ID": "str",
        "String": "str",
        "Int": "int",
        "Boolean": "bool",
        "GenericScalar": "Dict",
    }

    additional_bases = {}



from turms.config import GeneratorConfig
from turms.parser.imports import generate_imports
from turms.plugins.structure import StructurePlugin
from turms.processor.base import Processor
from turms.processor.black import BlackProcessor
from turms.utils import build_schema, introspect
from turms.plugins.base import Plugin
from typing import Dict, List
from turms.plugins.enums import EnumsPlugin
from turms.plugins.fragments import FragmentsPlugin
from turms.plugins.operation import OperationsPlugin
from turms.plugins.funcs import OperationsFuncPlugin
import ast

plugins: List[Plugin] = [
    EnumsPlugin(),
    FragmentsPlugin(),
    OperationsPlugin(),
    OperationsFuncPlugin(),
    StructurePlugin(),
]

processors: List[Processor] = [
    BlackProcessor(),
]


class GenerationError(Exception):
    pass


def generate(
    config: GeneratorConfig,
    introspection_query: Dict[str, str],
    plugins=plugins,
    processors=processors,
):

    client_schema = build_schema(introspection_query)

    global_tree = []

    global_tree += generate_imports(client_schema, config)

    for plugin in plugins:
        try:
            global_tree += plugin.generate_imports(client_schema, config)
        except Exception as e:
            raise GenerationError(f"Plugin Imports: {plugin} failed!") from e

    for plugin in plugins:
        try:
            global_tree += plugin.generate_body(client_schema, config)
        except Exception as e:
            raise GenerationError(f"Plugin Body:{plugin} failed!") from e

    md = ast.Module(body=global_tree, type_ignores=[])
    generated = ast.unparse(ast.fix_missing_locations(md))

    for processor in processors:
        generated = processor.run(generated)

    with open(config.generated_name, "w") as f:
        f.write(generated)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['turms',
 'turms.cli',
 'turms.parser',
 'turms.plugins',
 'turms.processor',
 'turms.wards']

package_data = \
{'': ['*']}

install_requires = \
['graphql-core>=3.2.0,<4.0.0', 'pydantic>=1.9.0,<2.0.0', 'rich>=11.0.0,<12.0.0']

entry_points = \
{'console_scripts': ['turms = turms.cli.main:entrypoint']}

setup_kwargs = {
    'name': 'turms',
    'version': '0.1.2',
    'description': 'graphql-codegen powered by pydantic',
    'long_description': '# turms\n\n[![codecov](https://codecov.io/gh/jhnnsrs/turms/branch/master/graph/badge.svg?token=UGXEA2THBV)](https://codecov.io/gh/jhnnsrs/turms)\n\n### DEVELOPMENT\n\n# Insipiration\n\nBaldr is a pure python implementation of the awesome graphql-codegen library, following a simliar extensible design.\nIt makes heavy use of pydantic and its serialization capablities and provides fully typed querys, mutations and subscriptions\n\n## Supports\n\n- Documents\n- Fragments\n- Enums\n- Query Functions\n\n## Featurs\n\n- Fully Modular (agnostic of graphql engine)\n- Specify type mixins, baseclasses...\n- Fully Support type hints for variables (Pylance, VSCode)\n\n## Installation\n\n```bash\npip install turms\n```\n\n## Usage\n\nOpen your workspace (create a virtual env), in the root folder\n\n```bash\nturms init\n```\n\nThis creates a configuration file in the working directory, edit this to reflect your\nsettings (see Configuration)\n\n```bash\nturms gen\n```\n\nGenerate beautifully typed Operations, Enums,...\n\n## Why Turms\n\nIn Etruscan religion, Turms (usually written as 𐌕𐌖𐌓𐌌𐌑 Turmś in the Etruscan alphabet) was the equivalent of Roman Mercury and Greek Hermes, both gods of trade and the messenger god between people and gods.\n',
    'author': 'jhnnsrs',
    'author_email': 'jhnnsrs@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import argparse
from enum import Enum
import os
from rich.console import Console

directory = os.getcwd()


class TurmsOptions(str, Enum):
    GEN = "gen"
    INIT = "init"
    DOWNLOAD = "download"


default_settings = """
    arkitekt:
        schema: http://localhost:3000/graphql
        out_dir: arkitekt
        plugins: 
            - type: turms.plugins.enums.EnumsPlugin
            - type: turms.plugins.fragments.FragmentsPlugin
              fragment_bases: 
                    - turms.graphql.GraphQLObject

"""


def main(script: TurmsOptions, path: str):

    console = Console()

    if path == ".":
        app_directory = os.getcwd()
        name = os.path.basename(app_directory)
    else:
        app_directory = os.path.join(os.getcwd(), path)
        name = path

    if script.INIT:
        console.log(f"Creating balrd.yaml in {app_directory}")
        with open(os.path.join(app_directory, "baldr.yaml"), "w") as f:
            f.write(default_settings)

    if script.GEN:
        console.log("[red]Error")


def entrypoint():
    parser = argparse.ArgumentParser(description="Say hello")
    parser.add_argument("script", type=TurmsOptions, help="The Script Type")
    parser.add_argument("path", type=str, help="The Path", nargs="?", default=".")
    parser.add_argument(
        "--services",
        type=str,
        help="The services you want to connect to (seperated by ,)",
    )
    args = parser.parse_args()

    main(
        script=args.script,
        path=args.path,
    )


if __name__ == "__main__":
    entrypoint()
